# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "Manager"
"""
  * @File    :   Manager.py
  * @Time    :   2024/11/06 20:24:27
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""
import re
from re import RegexFlag
from pathlib import Path
import os
import shutil
from typing import Callable, Any
from xy_console.utils import *

from .Object.File import File
from .types import FileType
from .Operation.Rename import Rename
from .Operation.Search import Search


class Manager:

    def rglob_clean(
        self,
        keyword: str | None = None,
        work_dir: Path = Path.cwd(),
        file_type: FileType = FileType.ANY,
        ignore_hidden_files: bool = True,
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
        should_delete_delegate: Callable[[list[Path]], bool] | None = None,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt

        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")
        if not keyword:
            raise ValueError(
                "请传入关键字(-r)的参数进行过滤, -r支持Linux文件名通配符搜索",
            )
        file_path_list = []
        invalidate_file_path_list = []
        search_result = (
            list(work_dir.rglob(keyword))
            if isinstance(keyword, str)
            and isinstance(work_dir, Path)
            and work_dir.exists()
            else []
        )
        if (
            not search_result
            or not isinstance(search_result, list)
            or len(search_result) == 0
        ):
            log_info("暂无匹配文件或目录")
            return
        file_path_list = []
        invalidate_file_path_list = []
        for work_glob in search_result:
            search: Search = Search()
            search.keyword = keyword
            search.work_dir = work_dir
            search.file_type = file_type
            search.ignore_hidden_files = ignore_hidden_files
            validated_path = search.validated_path(work_glob)
            if isinstance(validated_path, Path):
                file_verbose_name = FileType.verbose_name(validated_path)
                msg = f"将要删除... => [{file_verbose_name}] {validated_path}"
                log_info(msg)
                file_path_list.append(validated_path)
            else:
                invalidate_file_path_list.append(validated_path)
        if not file_path_list:
            log_info("暂无匹配文件或目录")
            return
        validate_delegate = should_delete_delegate != None and callable(
            should_delete_delegate
        )
        should_delete = (
            validate_delegate and should_delete_delegate(file_path_list) == True
        )
        if should_delete == True:
            log_info("开始删除...")
            success_file_path_list = []
            fail_file_path_list = []
            for file_path in file_path_list:
                try:
                    file_verbose_name = FileType.verbose_name(file_path)
                    log_info(f"正在删除... => [{file_verbose_name}] {file_path} ")
                    if file_path.is_dir():
                        shutil.rmtree(file_path)
                    else:
                        file_path.unlink()
                    success_file_path_list.append(file_path)
                except:
                    fail_file_path_list.append(file_path)
            if not fail_file_path_list:
                log_success("全部删除完成 !!!")
            else:
                for file_path in fail_file_path_list:
                    file_verbose_name = FileType.verbose_name(file_path)
                    msg = f"还未删除... => [{file_verbose_name}] {file_path}"
                    log_error(msg)
                log_error(
                    f"还剩下以上文件或目录未删除(共计: {len(fail_file_path_list) if isinstance(fail_file_path_list, list) else 0}): ==>"
                )
                log_error("=" * 50)
        else:
            log_info("取消删除...")

    def search_clean(
        self,
        keyword: str | None = None,
        work_dir: Path = Path.cwd(),
        file_type: FileType = FileType.ANY,
        ignore_hidden_files: bool = True,
        deep_start: int = 0,  # 0 => 绝对路径所有, -1 => 最后文件名.
        deep_count: int = 0,  # -1 => 递归所有, 0 => 仅当前, 1 => 当前+1级, 2 => 当前+2级, ...
        regex_flag: RegexFlag = re.M,
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
        should_delete_delegate: Callable[[list[Path]], bool] | None = None,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt

        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")
        if not keyword:
            raise ValueError(
                "请传入关键字(-k)的参数进行过滤, -k支持正则表达式匹配符进行搜索",
            )
        file_path_list = []
        invalidate_file_path_list = []
        search_result = list(work_dir.rglob("**/*"))
        file_path_list = []
        invalidate_file_path_list = []
        for work_glob in search_result:
            search: Search = Search()
            search.keyword = keyword
            search.work_dir = work_dir
            search.file_type = file_type
            search.ignore_hidden_files = ignore_hidden_files
            search.deep_start = deep_start
            search.deep_count = deep_count
            search.regex_flag = regex_flag
            validated_path = search.validated_path(work_glob)
            if (
                isinstance(validated_path, Path)
                and search.search(validated_path) == True
            ):
                file_verbose_name = FileType.verbose_name(validated_path)
                msg = f"将要删除... => [{file_verbose_name}] {validated_path}"
                log_info(msg)
                file_path_list.append(validated_path)
            else:
                invalidate_file_path_list.append(validated_path)
        if not file_path_list:
            log_info("暂无匹配文件或目录")
            return
        validate_delegate = should_delete_delegate != None and callable(
            should_delete_delegate
        )
        should_delete = (
            validate_delegate and should_delete_delegate(file_path_list) == True
        )
        if should_delete == True:
            log_info("开始删除...")
            success_file_path_list = []
            fail_file_path_list = []
            for file_path in file_path_list:
                try:
                    file_verbose_name = FileType.verbose_name(file_path)
                    log_info(f"正在删除... => [{file_verbose_name}] {file_path} ")
                    if file_path.is_dir():
                        shutil.rmtree(file_path)
                    else:
                        file_path.unlink()
                    success_file_path_list.append(file_path)
                except:
                    fail_file_path_list.append(file_path)
            if not fail_file_path_list:
                log_success("全部删除完成 !!!")
            else:
                for file_path in fail_file_path_list:
                    file_verbose_name = FileType.verbose_name(file_path)
                    msg = f"还未删除... => [{file_verbose_name}] {file_path}"
                    log_error(msg)
                log_error(
                    f"还剩下以上文件或目录未删除(共计: {len(fail_file_path_list) if isinstance(fail_file_path_list, list) else 0}): ==>"
                )
                log_error("=" * 50)
        else:
            log_info("取消删除...")

    def rglob_files(
        self,
        keyword: str | None = None,
        work_dir: Path = Path.cwd(),
        file_type: FileType = FileType.ANY,
        ignore_hidden_files: bool = True,
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt

        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")
        if not keyword:
            raise ValueError(
                "请传入关键字(-r)的参数进行过滤, -r支持Linux文件名通配符搜索",
            )
        file_path_list = []
        search_result = (
            list(work_dir.rglob(keyword))
            if isinstance(keyword, str)
            and isinstance(work_dir, Path)
            and work_dir.exists()
            else []
        )
        if (
            not search_result
            or not isinstance(search_result, list)
            or len(search_result) == 0
        ):
            log_info("暂无匹配对象")
            return
        file_path_list = []
        for work_glob in search_result:
            search: Search = Search()
            search.keyword = keyword
            search.work_dir = work_dir
            search.file_type = file_type
            search.ignore_hidden_files = ignore_hidden_files
            validated_path = search.validated_path(work_glob)
            if isinstance(validated_path, Path):
                file_verbose_name = FileType.verbose_name(validated_path)
                msg = f"查找到... => [{file_verbose_name}] {validated_path}"
                file_path_list.append(validated_path)
                log_info(msg)
        msg = f"匹配到 {len(file_path_list)} 个对象"
        if len(file_path_list) > 0:
            log_info(msg)
        else:
            log_info("暂无匹配对象")

    def search_files(
        self,
        keyword: str | None = None,
        work_dir: Path = Path.cwd(),
        file_type: FileType = FileType.ANY,
        ignore_hidden_files: bool = True,
        deep_start: int = 0,  # 0 => 绝对路径所有, -1 => 最后文件名.
        deep_count: int = -1,  # -1 => 递归所有, 0 => 仅当前, 1 => 当前+1级, 2 => 当前+2级, ...
        regex_flag: RegexFlag = re.M,
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt

        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")
        if not keyword:
            raise ValueError(
                "请传入关键字(-k)的参数进行过滤, -k支持正则表达式匹配符进行搜索",
            )
        search_result = work_dir.rglob("**/*")
        file_path_list = []
        for result in search_result:
            search: Search = Search()
            search.keyword = keyword
            search.work_dir = work_dir
            search.file_type = file_type
            search.ignore_hidden_files = ignore_hidden_files
            search.deep_start = deep_start
            search.deep_count = deep_count
            search.regex_flag = regex_flag
            validated_path = search.validated_path(result)
            if (
                isinstance(validated_path, Path)
                and search.search(validated_path) == True
            ):
                file_verbose_name = FileType.verbose_name(validated_path)
                msg = f"查找到... => [{file_verbose_name}] {validated_path}"
                log_info(msg)
                file_path_list.append(validated_path)
        if not file_path_list:
            log_info("暂无匹配文件或目录")
            return

    def rename_files(
        self,
        work_dir: Path = Path.cwd(),
        source_pattern: str | None = None,
        replace: str | Callable[[Any], str] | None = None,
        deep_start: int = 0,  # 0 => 绝对路径所有, -1 => 最后文件名.
        deep_count: int = -1,  # -1 => 递归所有, 0 => 仅当前, 1 => 当前+1级, 2 => 当前+2级, ...
        count: int = 0,  # 匹配次数, 0 => 无限制, 其他按输入次数为匹配次数
        file_type: FileType = FileType.ANY,
        ignore_hidden_files: bool = True,
        regex_flag: RegexFlag = re.M,
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
        should_rename_delegate: Callable[[dict[str, str]], bool] | None = None,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt

        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")
        if not source_pattern:
            raise ValueError(
                "请传入 源文件表达式 (-s) 的参数进行过滤, -s支持正则表达式匹配符进行搜索",
            )
        if not replace:
            raise ValueError(
                "请传入 目标文件表达式 (-t) 的参数进行过滤, -s支持正则表达式匹配符进行搜索",
            )
        search_result = work_dir.rglob("**/*")
        result_map = {}
        for result in search_result:
            search: Search = Search()
            search.keyword = result.as_posix()
            search.work_dir = work_dir
            search.file_type = file_type
            search.ignore_hidden_files = ignore_hidden_files
            search.deep_start = deep_start
            search.deep_count = deep_count
            search.regex_flag = regex_flag
            validated_path = search.validated_path(result)
            if (
                isinstance(validated_path, Path)
                and search.search(validated_path) == True
            ):
                rename: Rename = Rename()
                rename.file_type = file_type
                rename.ignore_hidden_files = ignore_hidden_files
                rename.regex_flag = regex_flag
                validated_path = rename.validated_path(result)
                if isinstance(validated_path, Path):
                    file_verbose_name = FileType.verbose_name(validated_path)
                    new_result = rename.rename(
                        result,
                        source_pattern=source_pattern,
                        replace=replace,
                        deep_start=deep_start,
                        deep_count=deep_count,
                        count=count,
                    )
                    if isinstance(new_result, Path) and new_result != result:
                        msg = f"匹配到... => [{file_verbose_name}] {validated_path}\n修改为: ==> \n{new_result}"
                        log_info(msg)
                        result_map.update(
                            {
                                validated_path.as_posix(): new_result.as_posix(),
                            }
                        )
        if not result_map:
            log_info("暂无匹配对象")
            return
        validate_delegate = should_rename_delegate != None and callable(
            should_rename_delegate
        )
        should_rename = validate_delegate and should_rename_delegate(result_map) == True
        if should_rename == True:
            success_map = {}
            fail_map = {}
            for source, target in result_map.items():
                try:
                    cmd = f"mv {source} {target}"
                    result = os.system(cmd)
                    if result == 0:
                        success_map.update({source: target})
                    else:
                        fail_map.update({source: target})
                except:
                    fail_map.update({source: target})
        else:
            log_info("取消重命名")

    def random_touch(
        self,
        work_dir: Path = Path.cwd(),
        extension: str | None = "",
        locale: str = "en",
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
        should_touch_delegate: Callable[[Path], bool] | None = None,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt

        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")
        file_path = File.random_filename(
            work_dir=work_dir,
            extension=extension,
            locale=locale,
        )
        msg = f"生成新文件路径为:  ==> {file_path} "
        log_info(msg)
        validate_delegate = should_touch_delegate != None and callable(
            should_touch_delegate
        )
        should_touch = validate_delegate and should_touch_delegate(file_path) == True
        if should_touch == True:
            result = File.touch(file_path)
            if isinstance(result, Path) and result.exists():
                log_info(f"创建文件: \n{file_path} \n成功!!!")
            else:
                log_info(f"创建文件: \n{file_path} \n失败!!!")
        else:
            log_info("取消创建文件")

    def random_mkdir(
        self,
        work_dir: Path = Path.cwd(),
        extension: str | None = "",
        locale: str = "en",
        log_info=printt,
        log_error=print_e,
        log_success=print_s,
        should_mkdir_delegate: Callable[[Path], bool] | None = None,
    ):
        if not log_info or not callable(log_info):
            log_info = printt
        if not log_error or not callable(log_error):
            log_error = printt
        if not log_success or not callable(log_success):
            log_success = printt
        if not os.access(work_dir, os.W_OK):  # type: ignore
            raise PermissionError(f"当前工作目录\n({work_dir})\n没有写入权限!!!")

        file_path = File.random_filename(
            work_dir=work_dir,
            extension=extension,
            locale=locale,
        )
        msg = f"生成新目录路径为:  ==> {file_path} "
        log_info(msg)
        validate_delegate = should_mkdir_delegate != None and callable(
            should_mkdir_delegate
        )
        should_mkdir = validate_delegate and should_mkdir_delegate(file_path) == True
        if should_mkdir == True:
            result = File.mkdir(file_path)
            if isinstance(result, Path) and result.exists():
                log_info(f"创建目录: \n{file_path} \n成功!!!")
            else:
                log_info(f"创建目录: \n{file_path} \n失败!!!")
        else:
            log_info("取消创建目录")



# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "Search"
"""
  * @File    :   Search.py
  * @Time    :   2024/11/06 20:15:24
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""

import re
from re import RegexFlag
from pathlib import Path
from xy_file.types import FileType


class Search:
    file_type: FileType = FileType.ANY
    work_dir: Path = Path.cwd()
    keyword: str = ""
    ignore_hidden_files: bool = True
    regex_flag: RegexFlag = re.M
    deep_start: int = 0
    # deep_start 0 => 绝对路径所有, -1 => 最后文件名.
    deep_count: int = -1
    # deep_count -1 => 递归所有, 0 => 仅当前, 1 => 当前+1级, 2 => 当前+2级, ...

    def validated_path(self, path: Path) -> Path | None:
        path_validate = self.file_type.validate(path=path)
        hidden_file_part = [part for part in path.parts if part.startswith(".")]
        is_hidden_file = not hidden_file_part or len(hidden_file_part) == 0
        ok = path_validate and not (
            is_hidden_file and self.ignore_hidden_files == False
        )
        if ok and path.exists():
            return path
        else:
            return None

    def deeped_path(self, path: Path) -> str:
        path_parts = None
        if self.deep_start == 0:
            path_parts = path.parts
        elif self.deep_start == -1:
            path_parts = path.parts[-1:]
        else:
            path_parts = path.parts[self.deep_start : self.deep_start + self.deep_count]
        if len(path_parts) > 0:
            path_str = "/".join(path_parts)
            if path_parts[0] == "/":
                path_str = path_str[1:]
            return path_str
        return ""

    def pattern(self) -> re.Pattern:
        return re.compile(
            self.keyword,
            flags=self.regex_flag,
        )

    def search(
        self,
        path: Path,
    ) -> bool:
        pattern = self.pattern()
        validated_path = self.validated_path(path)
        if isinstance(pattern, re.Pattern) and isinstance(validated_path, Path):
            deeped_path = self.deeped_path(validated_path)
            results = pattern.search(deeped_path)
            if results != None:
                return True
        return False



# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "Runner"
"""
  * @File    :   Runner.py
  * @Time    :   2024/11/05 21:42:24
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""
import re
from argparse import Namespace
from pathlib import Path

import xy_file
from xy_argparse.ArgParse import ArgParse
from xy_console.utils import print_e, print_s, inputt, print_r, print_w

from .types import FileType
from .Manager import Manager
from .Object.File import File


class Runner(ArgParse):
    manager = Manager()

    def __init__(
        self,
        *args,
    ):
        self.quick_default_info(xy_file.__name__)
        self.description = "文件管理应用"

    def main(self):
        self.default_parser()
        self.add_arguments()
        self.run_arguments()

    def add_arguments(self):
        self.add_argument(
            "-w",
            "--work",
            help_text="""
                命令:
                clean | c => 常用清理文件或目录,通配符匹配, 配合 -r/--rglob-keyword 使用, 默认 **/*, 表示当前目录下所有匹配的文件和目录;
                search_clean | sc => 正则表达式搜索文件或目录来清理文件或目录, 配合 -k/--keyword 使用, 必须符合正则表达式;
                rglob_files | rf => 通配符匹配文件或目录, 配合 -r/--rglob-keyword 使用, 默认 **/*, 表示当前目录下所有匹配的文件和目录;
                search_files | sf => 正则表达式搜索文件或目录,, 配合 -k/--keyword 使用, 必须符合正则表达式;
                rename_files | r => 正则表达式匹配来重命名文件或目录, 配合 -k/--keyword 使用, 必须符合正则表达式;
                random_touch | rt => 创建随机文件;
                random_mkdir | rm => 创建随机目录, 默认会生成带后缀的文件名, 使用 (xy_file -w rm -e "") 来创建不带后缀的随机目录;
                random_file_name | rfn => 生成随机文件路径;
            """,
            required=True,
        )
        self.add_argument(
            "-y",
            "--yes",
            type_name=bool,
            help_text="随便填写-y开头的参数如: -yy.就会强制运行, 所有过程遇到提示都默认为Yes;",
            default=False,
        )
        self.add_argument(
            "-d",
            "--work-dir",
            help_text="工作目录, 默认当前工作目录, Path.cwd();",
        )
        self.add_argument(
            "-r",
            "--rglob-keyword",
            help_text="""
            通配符
            工作目录过滤匹配关键字; 
            默认: **/*, 当前目录下的全部文件和目录，
            可选，工作目录过滤关键字，使用通配符进行匹配，例如 *.py表示当前目录下所有py脚本""",
            default="**/*",
        )
        self.add_argument(
            "-k",
            "--keyword",
            help_text="关键字; 可选; 使用正则表达式Search, Rename; 默认: re.M模式, !!! 注意: 该关键字匹配是绝对路径字符串",
        )
        self.add_argument(
            "-regf",
            "--regex-flag",
            type_name=int,
            help_text="匹配模式: 2 => re.I, 8 => re.M, 16 => re.S, 32 => re.U, 64 => re.X",
            default=re.M.value,
        )
        self.add_argument(
            "-i",
            "--ignore-hidden-files",
            type_name=bool,
            help_text="""是否忽略隐藏目录和文件""",
            default=1,
        )
        self.add_argument(
            "-f",
            "--file-type-filter",
            type_name=int,
            help_text="""
                            过滤器, 默认为0; 无限制,
                            !!! 若输入字符串则无大小写限制,
                            以下为过滤器对应类型:
                            任意 = [0, any],
                            文件 = [1, file],
                    目录/文件夹 = [2, dir],
                        软链接 = [3, symlink],
                        硬链接 = [4, hardlink],
                        Socket = [5, socket],
            字符设备(char device) = [6, char_device],
            块设备(block device) = [7, block_device],
                    管道(fifo) = [8, fifo],
                            """,
            default=0,
        )
        self.add_argument(
            "-e",
            "--extension",
            help_text="文件后缀, 用于随机生成文件或目录",
        )
        self.add_argument(
            "-l",
            "--locale",
            help_text="语言环境, 默认英文 => en, 简体中文 => zh_CN",
        )
        self.add_argument(
            "-ds",
            "--deep-start",
            type_name=int,
            help_text="路径深度:  0 => 绝对路径所有, -1 => 最后文件名.",
            default=0,
        )
        self.add_argument(
            "-dc",
            "--deep-count",
            type_name=int,
            help_text="递归深度:  -1 => 递归所有, 0 => 仅当前, 1 => 当前+1级, 2 => 当前+2级, ...",
            default=-1,
        )
        self.add_argument(
            "-t",
            "--count",
            type_name=int,
            help_text="匹配次数, 0 => 无限制, 其他按输入次数为匹配次数",
            default=0,
        )
        self.add_argument(
            "-sp",
            "--source-pattern",
            help_text="源匹配式, 需符合正则表达式",
        )
        self.add_argument(
            "-rp",
            "--replace",
            help_text="目标匹配式, 需符合正则表达式",
        )

    @property
    def yes(self) -> bool:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            yes = arguments.yes
            if yes == None:
                return False
            return yes
        return False

    @property
    def work(self):
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.work
        return None

    @property
    def work_dir(self) -> str | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.work_dir
        return None

    @property
    def rglob_keyword(self):
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.rglob_keyword
        return None

    @property
    def keyword(self):
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.keyword
        return None

    @property
    def regex_flag(self):
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.regex_flag
        return re.M.value

    @property
    def ignore_hidden_files(self):
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.ignore_hidden_files
        return True

    @property
    def file_type_filter(self):
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.file_type_filter
        return FileType.ANY.value

    @property
    def extension(self) -> str | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.extension
        return None

    @property
    def locale(self) -> str | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.locale
        return "en"

    @property
    def deep_start(self) -> int | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.deep_start
        return 0

    @property
    def deep_count(self) -> int | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.deep_count
        return -1

    @property
    def count(self) -> int | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.count
        return 0

    @property
    def source_pattern(self) -> str | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.source_pattern
        return None

    @property
    def replace(self) -> str | None:
        arguments = self.arguments()
        if isinstance(arguments, Namespace):
            return arguments.replace
        return None

    def on_arguments(self, name, value, arguments=None):
        rglob_keyword = self.rglob_keyword
        keyword = self.keyword
        file_type_filter = self.file_type_filter
        file_type = FileType.parse(file_type_filter)
        ignore_hidden_files = bool(self.ignore_hidden_files)
        deep_start = self.deep_start if isinstance(self.deep_start, int) else 0
        deep_count = self.deep_count if isinstance(self.deep_count, int) else -1
        count = self.count if isinstance(self.count, int) else 0
        extension = self.extension
        yes = self.yes if isinstance(self.yes, bool) else False
        regex_flag = (
            re.RegexFlag(self.regex_flag) if isinstance(self.regex_flag, int) else re.M
        )
        source_pattern = (
            self.source_pattern
            if isinstance(self.source_pattern, str) and self.source_pattern
            else self.source_pattern
        )
        replace = (
            self.replace
            if isinstance(self.replace, str) and self.replace
            else self.replace
        )
        locale = self.locale if isinstance(self.locale, str) else "en"
        work_dir = self.work_dir
        if work_dir == None:
            work_dir = Path.cwd()
        if not isinstance(work_dir, Path):
            work_dir = Path(work_dir)
        if name == "work":
            if value == "random_file_name" or value == "rfn":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 随机生成文件名失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return
                randome_file_name = File.random_filename(
                    work_dir=work_dir,
                    extension=extension,
                )
                print_s(f"生成新的随机文件路径 => {randome_file_name}")
                print()
            elif value == "random_touch" or value == "rt":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 随机生成文件失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return

                def should_touch_delegate(path: Path) -> bool:
                    if yes == True:
                        return True
                    print_w("是否创建以上文件?")
                    should_touch = inputt(
                        "确定请输入Y, 输入其他为取消创建命令. 请输入[Y/n]\n"
                    )
                    if should_touch == "Y":
                        return True
                    return False

                self.manager.random_touch(
                    Path.cwd(),
                    extension=extension,
                    locale=locale if locale else "en",
                    should_touch_delegate=should_touch_delegate,
                )
            elif value == "random_mkdir" or value == "rm":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 随机生成目录失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return

                def should_mkdir_delegate(path: Path) -> bool:
                    if yes == True:
                        return True
                    print_w("是否创建以上目录?")
                    should_mkdir = inputt(
                        "确定请输入Y, 输入其他为取消创建命令. 请输入[Y/n]\n"
                    )
                    if should_mkdir == "Y":
                        return True
                    return False

                self.manager.random_mkdir(
                    work_dir,
                    extension=extension,
                    locale=locale if locale else "en",
                    should_mkdir_delegate=should_mkdir_delegate,
                )
            elif value == "rglob_files" or value == "rf":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 通配符匹配搜索文件失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return
                self.manager.rglob_files(
                    keyword=rglob_keyword,
                    work_dir=work_dir,
                    file_type=file_type,
                    ignore_hidden_files=ignore_hidden_files,
                )
            elif value == "search_files" or value == "sf":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 正则表达式搜索文件失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return
                self.manager.search_files(
                    keyword=keyword,
                    work_dir=work_dir,
                    file_type=file_type,
                    ignore_hidden_files=ignore_hidden_files,
                    deep_start=deep_start,
                    deep_count=deep_count,
                    regex_flag=regex_flag,
                )
            elif value == "rename_files" or value == "r":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 重命名文件失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return

                def should_rename_delegate(rename_map: dict[str, str]) -> bool:
                    if yes == True:
                        return True
                    print_w("是否重命名以上目录?")
                    should_mkdir = inputt(
                        "确定请输入Y, 输入其他为取消重命名命令. 请输入[Y/n]\n"
                    )
                    if should_mkdir == "Y":
                        return True
                    return False

                self.manager.rename_files(
                    work_dir=work_dir,
                    source_pattern=source_pattern,
                    replace=replace,
                    deep_start=deep_start,
                    deep_count=deep_count,
                    count=count,
                    file_type=file_type,
                    ignore_hidden_files=ignore_hidden_files,
                    should_rename_delegate=should_rename_delegate,
                )
            elif value == "search_clean" or value == "sc":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 搜索文件失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return

                def should_delete_delegate(path_list: list[Path]) -> bool:
                    if yes == True:
                        return True
                    print_w("是否清理以上对象?")
                    should_mkdir = inputt(
                        "确定请输入Y, 输入其他为取消清理命令. 请输入[Y/n]\n"
                    )
                    if should_mkdir == "Y":
                        return True
                    return False

                self.manager.search_clean(
                    keyword=keyword,
                    work_dir=work_dir,
                    file_type=file_type,
                    ignore_hidden_files=ignore_hidden_files,
                    deep_start=deep_start,
                    deep_count=deep_count,
                    regex_flag=regex_flag,
                    should_delete_delegate=should_delete_delegate,
                )
            elif value == "clean" or value == "c":
                if Path(work_dir).exists() == False:
                    print_e(
                        "工作目录不存在, 搜索文件失败, 请传入-d/--work-dir参数为工作目录"
                    )
                    return

                def should_delete_delegate(path_list: list[Path]) -> bool:
                    if yes == True:
                        return True
                    print_w("是否清理以上对象?")
                    should_mkdir = inputt(
                        "确定请输入Y, 输入其他为取消清理命令. 请输入[Y/n]\n"
                    )
                    if should_mkdir == "Y":
                        return True
                    return False

                self.manager.rglob_clean(
                    keyword=rglob_keyword,
                    work_dir=work_dir,
                    file_type=file_type,
                    ignore_hidden_files=ignore_hidden_files,
                    should_delete_delegate=should_delete_delegate,
                )



# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "Permission"
"""
  * @File    :   Permission.py
  * @Time    :   2024/11/06 20:27:11
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""


class Permission:
    pass



# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "types"
"""
  * @File    :   types.py
  * @Time    :   2024/11/06 20:22:05
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""

from enum import IntEnum
from pathlib import Path


class FileType(IntEnum):
    ANY = 0
    FILE = 1
    DIR = 2
    SYMLINK = 3
    HARDLINK = 4
    SOCKET = 5
    CHAR_DEVICE = 6
    BLOCK_DEVICE = 7
    FIFO = 8

    @classmethod
    def parse(cls, file_type_filter: str | int | None):
        if not file_type_filter:
            return FileType.ANY

        names = [member for member in cls if member.name.upper() == file_type_filter]
        if names and len(names) > 0:
            return names[0]

        values = [member for member in cls if member.value == file_type_filter]
        if values and len(values) > 0:
            return values[0]

        return FileType.ANY

    def validate(self, path: Path) -> bool:
        if not path or not path.exists():
            return False
        match self:
            case FileType.ANY:
                return True
            case FileType.FILE:
                return path.is_file()
            case FileType.DIR:
                return path.is_dir()
            case FileType.SYMLINK:
                return path.is_symlink()
            case FileType.HARDLINK:
                return path.stat().st_nlink == 2
            case FileType.SOCKET:
                return path.is_socket()
            case FileType.CHAR_DEVICE:
                return path.is_char_device()
            case FileType.BLOCK_DEVICE:
                return path.is_block_device()
            case FileType.FIFO:
                return path.is_fifo()
            case _:
                return True

    @staticmethod
    def verbose_name(file_path: Path) -> str:
        file_type = "文件"
        if file_path.is_dir():
            file_type = "目录"
        elif file_path.is_symlink():
            file_type = "软链接"
        elif file_path.stat().st_nlink == 2:
            file_type = "硬连接"
        elif file_path.is_socket():
            file_type = "Socket"
        elif file_path.is_char_device():
            file_type = "字符设备(char_device)"
        elif file_path.is_block_device():
            file_type = "块设备(block_device)"
        elif file_path.is_fifo():
            file_type = "管道"
        else:
            file_type = "文件"
        return file_type



# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "utils"
"""
  * @File    :   utils.py
  * @Time    :   2023/06/03 22:23:42
  * @Author  :   余洋
  * @Version :   1.0
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   None
"""

from rich.console import Console

xy_c = Console()
xy_p = xy_c.print

_success = "bold #22ff00"
_execute = "bold #00ffff"
_warning = "bold #ffff00"
_run = "bold #2266ff"
_error = "bold #ff4500"
_red = "bold red"
_green = "bold green"
_blue = "bold blue"
_seperate_text = "\n======================================\n"


def inputt(
    text: str,
    prefix=_seperate_text,
    markup: bool = True,
    emoji: bool = True,
    password: bool = False,
    stream=None,
    *args,
    **kwargs,
):
    return xy_c.input(
        f"{prefix}{text}",
        markup=markup,
        emoji=emoji,
        password=password,
        stream=stream,
        *args,
        **kwargs,
    )


def printt(
    text: str,
    style: str = _run,
    prefix=_seperate_text,
    sep: str = " ",
    end: str = "\n",
    new_line_start: bool = False,
    crop: bool = True,
    justify=None,
    overflow=None,
    no_wrap=None,
    emoji=None,
    markup=None,
    highlight=None,
    width=None,
    height=None,
    soft_wrap=None,
) -> None:
    xy_p(
        f"{prefix}{text}",
        style=style,
        sep=sep,
        end=end,
        new_line_start=new_line_start,
        crop=crop,
        justify=justify,
        overflow=overflow,
        no_wrap=no_wrap,
        emoji=emoji,
        markup=markup,
        highlight=highlight,
        width=width,
        height=height,
        soft_wrap=soft_wrap,
    )


def print_s(
    text: str,
    prefix=_seperate_text,
):
    printt(
        text=text,
        prefix=prefix,
        style=_success,
    )


def print_exe(
    text: str,
    prefix=_seperate_text,
):
    printt(
        text=text,
        prefix=prefix,
        style=_execute,
    )


def print_w(
    text: str,
    prefix=_seperate_text,
):
    printt(
        text=text,
        prefix=prefix,
        style=_warning,
    )


def print_r(
    text: str,
    prefix=_seperate_text,
):
    printt(
        text=text,
        prefix=prefix,
        style=_run,
    )


def print_e(
    text: str,
    prefix=_seperate_text,
):
    printt(
        text=text,
        prefix=prefix,
        style=_error,
    )



# -*- coding: UTF-8 -*-
__author__ = "余洋"
__doc__ = "Rename"
"""
  * @File    :   Rename.py
  * @Time    :   2024/11/06 20:15:37
  * @Author  :   余洋
  * @Version :   0.0.1
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   
"""
import re
from re import RegexFlag
from typing import Any, Callable, List
from pathlib import Path
from xy_file.types import FileType


class Rename:

    file_type: FileType = FileType.ANY
    ignore_hidden_files: bool = True
    regex_flag: RegexFlag = re.M

    def validated_path(self, path: Path) -> Path | None:
        path_validate = self.file_type.validate(path=path)
        hidden_file_part = [part for part in path.parts if part.startswith(".")]
        is_hidden_file = not hidden_file_part or len(hidden_file_part) == 0
        ok = path_validate and not (
            is_hidden_file and self.ignore_hidden_files == False
        )
        if ok and path.exists():
            return path
        else:
            return None

    def rename(
        self,
        file_path: Path | str | None = None,
        source_pattern: str | None = None,
        replace: str | Callable[[Any], str] | None = None,
        deep_start: int = 0,  # 0 => 绝对路径所有, -1 => 最后文件名.
        deep_count: int = -1,  # -1 => 递归所有, 0 => 仅当前, 1 => 当前+1级, 2 => 当前+2级, ...
        count: int = 0,  # 匹配次数, 0 => 无限次, 其余按照输入数量为匹配次数
    ) -> Path | None:
        if file_path == None or source_pattern == None or replace == None:
            return None
        if isinstance(file_path, str):
            file_path = Path(file_path)
        file_path = self.validated_path(file_path)
        if file_path == None:
            return None
        if isinstance(file_path, Path):
            file_path_parts = None
            if deep_start == 0:
                file_path_parts = file_path.parts
            elif deep_start == -1:
                file_path_parts = file_path.parts[-1:]
            else:
                file_path_parts = file_path.parts[deep_start : deep_start + deep_count]
            if len(file_path_parts) > 0:
                file_path = "/".join(file_path_parts)
                if file_path_parts[0] == "/":
                    file_path = file_path[1:]
        result = re.sub(
            source_pattern,
            replace,
            file_path,
            count=count,
            flags=self.regex_flag,
        )
        if result == None:
            return None
        return Path(result)



# -*- coding: UTF-8 -*-
__author__ = "余洋"
"""
  * @File    :   File.py
  * @Time    :   2023/04/22 19:25:26
  * @Author  :   余洋
  * @Version :   1.0
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   None
"""

import logging
from pathlib import Path
from faker import Faker


class File:
    @staticmethod
    def random_filename(
        work_dir: Path,
        extension: str | None = None,
        locale="en",
    ) -> Path:
        faker = Faker(locale=locale)
        file_name = faker.file_name(extension=extension)
        file_path = work_dir.joinpath(file_name)
        if file_path.exists() == True:
            return File.random_filename(
                work_dir,
                extension=extension,
                locale=locale,
            )
        return file_path

    @staticmethod
    def random_touch(
        work_dir: Path,
        extension: str | None = None,
        locale="en",
    ) -> Path | None:
        file_path = File.random_filename(
            work_dir=work_dir,
            extension=extension,
            locale=locale,
        )
        return File.touch(file_path)

    @staticmethod
    def random_mkdir(
        work_dir: Path,
        extension: str | None = "",
        locale="en",
    ) -> Path | None:
        dir_path = File.random_filename(
            work_dir=work_dir,
            extension=extension,
            locale=locale,
        )
        return File.mkdir(dir_path)

    @staticmethod
    def touch(
        file_path: Path | str | None,
        force: bool = True,
    ) -> Path | None:
        if not file_path:
            return None

        if isinstance(file_path, str):
            file_path = Path(file_path)

        if isinstance(file_path, Path) and file_path.exists():
            return file_path

        try:
            if force:
                file_path.parent.mkdir(parents=True, exist_ok=True)
            file_path.touch()
            return file_path.resolve()
        except Exception as exception:
            logging.exception(exception)
            return None

    @staticmethod
    def mkdir(
        file_path: Path | str | None,
        parents: bool = True,
        exist_ok: bool = True,
    ) -> Path | None:
        if not file_path:
            return None

        if isinstance(file_path, str):
            file_path = Path(file_path)

        if isinstance(file_path, Path) and file_path.exists():
            return file_path
        try:
            file_path.mkdir(
                parents=parents,
                exist_ok=exist_ok,
            )
            return file_path.resolve()
        except Exception as exception:
            logging.exception(exception)
            return None
